﻿Global cpvzero.cpVect;

;==========================================================================================================================-
; This is another "helloword" by MadMedic
; It is some more complex, adding more objects and their visualization in 2D mode
;====================================================================================================-
InitSprite()
OpenWindow(0, 0, 0, 640, 480, "Chipmunk4PB wrapper example",  #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
OpenWindowedScreen(WindowID(0), 0, 0, 640, 480)

Global  *space.cpSpace = cpSpaceNew() ; Create an empty space.
                                      ;there is positive gravity - To safe screen coordinates orientation
Define gravity.cpVect : cpv(gravity, 0.0, 100) ; cpVect is a 2D vector And cpv() is a shortcut For initializing them.
cpSpaceSetGravity(*space, gravity)	

; Add a Static line segment shape For the ground.
; We'll make it slightly tilted so the ball will roll off.
; We attach it To space->staticBody To tell Chipmunk it shouldn't be movable.
A.cpVect 
B.cpVect 

;make borders
cpv(A, 0, 480) : cpv(B, 640, 480)
*ground.cpShape = cpSegmentShapeNew(*space\staticBody, A, B, 0)
cpShapeSetFriction(*ground, 1)
cpSpaceAddShape(*space, *ground)
;GetSizeOf()

cpv(A, 640, 0) : cpv(B, 640, 480)
*ground.cpShape = cpSegmentShapeNew(*space\staticBody, A, B, 0)
cpShapeSetFriction(*ground, 1)
cpSpaceAddShape(*space, *ground)

cpv(A, 0, 0) : cpv(B, 0, 480)
*ground.cpShape = cpSegmentShapeNew(*space\staticBody, A, B, 0)
cpShapeSetFriction(*ground, 1)
cpSpaceAddShape(*space, *ground)


Structure BodyStruct
  Moment.cpFloat
  *Body.cpBody
  *Shape.cpShape
  pos.cpVect
  Sprite.l
  Width.l
  Height.l
EndStructure

Global NewList BodyList.BodyStruct()

Procedure Body_CircleAdd(x.d, y.d, radius.d, mass)
  AddElement(BodyList())
  BodyList()\Moment = cpMomentForCircle(mass, 0, radius, cpvzero)
  BodyList()\Body = cpSpaceAddBody(*space, cpBodyNew(mass, BodyList()\moment))
  BodyPos.cpVect : cpv(BodyPos, x, y)
  cpBodySetPosition(BodyList()\Body, BodyPos)
  BodyList()\Shape = cpSpaceAddShape(*space, cpCircleShapeNew(BodyList()\Body, radius, cpvzero))
  cpShapeSetFriction(BodyList()\Shape, 0.25)
  
  BodyList()\Width = radius * 2
  BodyList()\Height = radius * 2
  BodyList()\sprite = CreateSprite(#PB_Any, BodyList()\Width, BodyList()\Height, #PB_Sprite_AlphaBlending)
  StartDrawing(SpriteOutput(BodyList()\sprite))
  DrawingMode(#PB_2DDrawing_AllChannels)
  Box(0,0, BodyList()\Width, BodyList()\Height, RGBA(Random(255,155), Random(255,155), Random(255,155),0))
  DrawingMode(#PB_2DDrawing_Transparent)
  DrawText(BodyList()\Width * 0.5 - TextWidth("Chipmunk") * 0.5, BodyList()\Height * 0.5 - TextHeight("Chipmunk") * 0.5, "Chipmunk", RGB(0,0,0))
  DrawingMode(#PB_2DDrawing_AlphaChannel)
  Circle(BodyList()\Width *0.5, BodyList()\Height *0.5, BodyList()\Width*0.5, RGBA(255,255,255, 255))
  StopDrawing()
  
EndProcedure
Procedure Body_BoxAdd(x, y, width, height, mass)
  AddElement(BodyList())
  BodyList()\Moment = cpMomentForBox(mass, width, height)
  BodyList()\Body = cpSpaceAddBody(*space, cpBodyNew(mass, BodyList()\moment))
  BodyPos.cpVect : cpv(BodyPos, x, y)
  cpBodySetPosition(BodyList()\Body, BodyPos)
  BodyList()\Shape = cpSpaceAddShape(*space, cpBoxShapeNew(BodyList()\Body, width, height, 0))
  cpShapeSetFriction(BodyList()\Shape, 0.25)
  
  BodyList()\Width = width
  BodyList()\Height = height
  BodyList()\sprite = CreateSprite(#PB_Any, BodyList()\Width, BodyList()\Height, #PB_Sprite_AlphaBlending)
  StartDrawing(SpriteOutput(BodyList()\sprite))
  DrawingMode(#PB_2DDrawing_AllChannels)
  Box(0,0, BodyList()\Width, BodyList()\Height, RGBA(Random(255,155), Random(255,155), Random(255,155),255))
  DrawingMode(#PB_2DDrawing_Transparent)
  DrawText(BodyList()\Width * 0.5 - TextWidth("Chipmunk") * 0.5, BodyList()\Height * 0.5 - TextHeight("Chipmunk") * 0.5, "Chipmunk", RGB(0,0,0))
  StopDrawing()
  
EndProcedure
Procedure Body_Update()
  Protected pos.cpVect
  Protected Angle.cpFloat
  
  ForEach BodyList()
    cpBodyGetPosition(pos, BodyList()\Body)
    Angle = Degree(cpBodyGetAngle(BodyList()\Body))
    RotateSprite(BodyList()\Sprite, Angle, #PB_Absolute)
    DisplayTransparentSprite(BodyList()\Sprite, pos\x - BodyList()\Width *0.5, pos\y - BodyList()\Height *0.5)
  Next
EndProcedure


Define i
For i=0 To 10
  Body_CircleAdd(Random(500,100), Random(200,100), Random(40,20), 1)
  Body_BoxAdd(Random(500,100), Random(200,100), Random(40,20), Random(40,20), 1)
Next 


; Now that it's all set up, we simulate all the objects in the space by
; stepping forward through time in small increments called steps.
; It is *highly* recommended To use a fixed size time Step.
Define timeStep.cpFloat = 1.0/60.0
Define Time.cpFloat

Define pos.cpVect, vel.cpVect
Define exit, event

Repeat
  ; It's very important to process all the events remaining in the queue at each frame
  ;
  exit = 0
  Repeat
    Event = WindowEvent()
    
    Select Event 
      Case #PB_Event_CloseWindow
        exit = 1
    EndSelect
  Until Event = 0
  
  If exit = 1
    Break
  EndIf
  
  cpSpaceStep(*space, 0.1)
  
  
  FlipBuffers() 
  ClearScreen(RGB(64, 128, 64))
  
  Body_Update()
  
  Delay(1)
ForEver


; Clean up our objects and exit!
; release sprites
ForEach BodyList()
  FreeSprite(BodyList()\Sprite)
Next

;ChipmunkFreeSpaceChildren (*space) ; use this function to remove and destroy all space objects, instead of cleaning them separately
cpShapeFree(*ballShape)
cpBodyFree(*ballBody)
cpShapeFree(*ground)
cpSpaceFree(*space)


Procedure  GetSizeOf()
  Debug "sizeof(cpArray)" + SizeOf(cpArray)
  Debug "sizeof(cpVect)" + SizeOf(cpVect)
  Debug "sizeof(cpTransform)" + SizeOf(cpTransform)
  Debug "sizeof(cpSpatialIndexClass)" + SizeOf(cpSpatialIndexClass)
  Debug "sizeof(cpSpatialIndex)" + SizeOf(cpSpatialIndex)
  Debug "sizeof(cpBB )" + SizeOf(cpBB )
  Debug "sizeof(cpBBTree)" + SizeOf(cpBBTree)
  Debug "sizeof(cpMat2x2 )" + SizeOf(cpMat2x2 )
  Debug "sizeof(cpSplittingPlane)" + SizeOf(cpSplittingPlane)
  Debug "sizeof(cpHashSetBin)" + SizeOf(cpHashSetBin)
  Debug "sizeof(cpHashSet)" + SizeOf(cpHashSet)
  Debug "sizeof(cpSweep1D)" + SizeOf(cpSweep1D)
  Debug "sizeof(cpBody  )" + SizeOf(cpBody  )
  Debug "sizeof(cpShapeFilter)" + SizeOf(cpShapeFilter)
  Debug "sizeof(cpShapeMassInfo)" + SizeOf(cpShapeMassInfo)
  Debug "sizeof(cpShapeClass)" + SizeOf(cpShapeClass)
  Debug "sizeof(cpShape)" + SizeOf(cpShape)
  Debug "sizeof(cpPolyShape)" + SizeOf(cpPolyShape)
  Debug "sizeof(cpCircleShape)" + SizeOf(cpCircleShape)
  Debug "sizeof(cpSegmentShape)" + SizeOf(cpSegmentShape)
  Debug "sizeof(cpCollisionHandler)" + SizeOf(cpCollisionHandler)
  Debug "sizeof(cpCollisionInfo)" + SizeOf(cpCollisionInfo)
  Debug "sizeof(cpConstraint)" + SizeOf(cpConstraint)
  Debug "sizeof(cpConstraintClass)" + SizeOf(cpConstraintClass)
  Debug "sizeof(cpGrooveJoint)" + SizeOf(cpGrooveJoint)
  Debug "sizeof(cpPinJoint)" + SizeOf(cpPinJoint)
  Debug "sizeof(cpPivotJoint)" + SizeOf(cpPivotJoint)
  Debug "sizeof(cpRotaryLimitJoint)" + SizeOf(cpRotaryLimitJoint)
  Debug "sizeof(cpRatchetJoint )" + SizeOf(cpRatchetJoint )
  Debug "sizeof(cpGearJoint)" + SizeOf(cpGearJoint)
  Debug "sizeof(cpSlideJoint)" + SizeOf(cpSlideJoint)
  Debug "sizeof(cpSimpleMotor)" + SizeOf(cpSimpleMotor)
  Debug "sizeof(cpSpace)" + SizeOf(cpSpace)
  Debug "sizeof(cpSpaceDebugColor)" + SizeOf(cpSpaceDebugColor)
  Debug "sizeof(cpSpaceHash)" + SizeOf(cpSpaceHash)
  Debug "sizeof(cpArbiterThread)" + SizeOf(cpArbiterThread)
  Debug "sizeof(cpArbiter)" + SizeOf(cpArbiter)
  Debug "sizeof(cpContactPoints)" + SizeOf(cpContactPoints)
  Debug "sizeof(cpContactPointSet)" + SizeOf(cpContactPointSet)
  Debug "sizeof(cpContact)" + SizeOf(cpContact)
  Debug "sizeof(cpDampedSpring)" + SizeOf(cpDampedSpring)
  Debug "sizeof(cpDampedRotarySpring)" + SizeOf(cpDampedRotarySpring)
  Debug "sizeof(cpPointQueryInfo)" + SizeOf(cpPointQueryInfo)
  Debug "sizeof(cpSegmentQueryInfo)" + SizeOf(cpSegmentQueryInfo)
  Debug "sizeof(cpContactPoints)" + SizeOf(cpContactPoints)
EndProcedure
; IDE Options = PureBasic 5.60 (Windows - x86)
; ExecutableFormat = Console
; Folding = -
; Markers = 1
; Executable = ..\HelloNewton2D_Chipmunk4PB.exe
; DisableDebugger
; Compiler = PureBasic 5.60 (Windows - x86)